//=============================================================================
//	MuseScore - Embellish plugin
//
// Preparation:	Adjust mingap = 480 (length of a quarter note) if desired. 
//				Embellishments are added halfway between existing symbols if in between space > mingap
//				If no note is found at this position, the previous melody note will be used unless 
//				there is yet another note before the next chord symbol, e.g. a quarter dotted note 
//				followed an eight note will use the 'upbeat' eight note as melody note and also put
//				the embellishments there.
//				Experiment with inversions for best voice leading.
//				For score in 3/4 adjust the onemeasure var.
//
// Limitations:
// Ubiquitous Circle of Fifths progressions are not analyzed
// Naked chords Bb etc are recognized as Maj; others: see embel table, adaptable to your symbols
// or adapt your symbols as below: search: // Adapt alternate chord symbols
//=============================================================================

import QtQuick 2.0
import MuseScore 3.0

MuseScore {
	version:	"0.2"
	description: "This plugin embellishes jazz lead sheet harmony"
	menuPath: "Plugins.Embellish"
	
	// sets cursor at time or next segment
	function setCursorToTime(cursor, time){
		cursor.rewind(0);
		while (cursor.segment) { 
			var current_time = cursor.tick;
			if(current_time>=time){
				return true;
			}
			cursor.next();
		}
		cursor.rewind(0);
		return false;
	}

	// sets cursor at time
    function setCursorToTime2(cursor, time){
		cursor.rewind(0);
		while (cursor.segment) { 
			var current_time = cursor.tick;
			if(current_time==time){
				return true;
			}
			cursor.next();
		}
		cursor.rewind(0);
		return false;
	}

	function findindex(thishead, heads){
	    for(var i=0; i<heads.length; i++){
			if(heads[i][0] == thishead){
				return heads[i][1];
			}
		}		
        
    }

	function finddimhead(index, dimheads){
		for(var i=0; i<dimheads.length; i++){
			if(dimheads[i][1] == index){
				return dimheads[i][0];
			}
		}			       
    }
	// tries to steer the slash chord in the proper #-b realm in harmony with the top chord
    // here is priorhead the top of the slashchord
	function findshead(priorhead,index, slashi2, heads, sharpheads, flatheads,slashhead){
		//console.log ("priorhead = " + priorhead)
		index++;
        for (var c= 0; c < slashhead.length; c++){
            if(slashhead[c][0] == priorhead){
                var type = slashhead[c][index];
            }
			
        }
		//console.log ("c = " + c + " type = " + type + " index = " + index)
		if (type == -1){
			//console.log ("in flatheads")
			for(var i=0; i<flatheads.length; i++){
				if(flatheads[i][1] == slashi2){
					return flatheads[i][0];
				}
			}	
		}	
		else if (type == 1){
			//console.log ("in sharpheads")
			for(var i=0; i<sharpheads.length; i++){
				if(sharpheads[i][1] == slashi2){
					return sharpheads[i][0];
				}
			}	
		}
		else {
			//console.log ("in heads")
			for(var i=0; i<heads.length; i++){
				if(heads[i][1] == slashi2){
					return heads[i][0];
				}
			}	
		}		    
    }
	
	// tries to steer the slash chord in the proper #-b realm in harmony with the previous chord
	// priorhead is head of previous chord
	function findhead(priorhead,index, heads, sharpheads, flatheads){
		if (priorhead.includes("b")== true){
			//console.log ("in flatheads")
			for(var i=0; i<flatheads.length; i++){
				if(flatheads[i][1] == index){
					return flatheads[i][0];
				}
			}	
		}	
		else if (priorhead.includes("#")== true){
			//console.log ("in sharpheads")
			for(var i=0; i<sharpheads.length; i++){
				if(sharpheads[i][1] == index){
					return sharpheads[i][0];
				}
			}	
		}
		else {
			//console.log ("in heads")
			for(var i=0; i<heads.length; i++){
				if(heads[i][1] == index){
					return heads[i][0];
				}
			}	
		}		    
    }
	
	function findheadlen(harmony){
		var str = harmony;
		var t = str.substr(1,1);
		if ((t == "b") || (t == "#")){
			return 2;
		}
		else{
			return 1;
		}
	}
	
	function findnewdistance(d1, d2){
		d1 = parseInt(d1);
		d2 = parseInt(d2);
		d1 = d1 + d2;
		//console.log("d1 = ", d1);
		if (d1 > 12) d1 -= 12;
		if (d1 < 1) d1 += 12;
		return d1;
	}

	function findDelta(d1, d2){
		d1 = parseInt(d1);
		d2 = parseInt(d2);
		d1 = d2 - d1;
		// //console.log("d1 = ", d1);
		if (d1 >= 12) d1 -= 12;
		if (d1 < 0) d1 += 12;
		return d1;
	}
	
	
	onRun: {

	var heads = [
["C",1],
["G",8],
["D",3],
["A",10],
["E",5],
["B",12],
["F#",7],
["Gb",7],
["Db",2],
["C#",2],
["Ab",9],
["G#",9],
["Eb",4],
["D#",4],
["Bb",11],
["A#",11],
["F",6],
];	
	var sharpheads = [
["C",1],
["G",8],
["D",3],
["A",10],
["E",5],
["B",12],
["F#",7],
["Gb",7],
["C#",2],
["Db",2],
["G#",9],
["Ab",9],
["D#",4],
["Eb",4],
["A#",11],
["Bb",11],
["F",6],
];

var flatheads = [
["C",1],
["F",6],
["Bb",11],
["A#",11],
["Eb",4],
["D#",4],
["Ab",9],
["G#",9],
["Db",2],
["C#",2],
["Gb",7],
["F#",7],
["B",12],
["E",5],
["A",10],
["D",3],
["G",8],
];

var dimheads = [
["C",1],
["D",3],
["E",5],
["F",6],
["G",8],
["A",10],
["B",12],
["Eb",4],
["Ab",9],
["Bb",11],
["C#",2],
["F#",7],
];
var slashhead = [
["C",0,-1,0,-1,0,0,-1,0,-1,0,-1,0],
["G",0,-1,0,-1,0,0,-1,0,-1,0,0,1],
["D",0,-1,0,0,1,0,-1,0,-1,0,0,1],
["A",0,-1,0,0,1,0,-1,0,0,0,0,1],
["E",0,0,1,0,1,0,-1,0,0,0,0,1],
["B",0,0,1,0,1,0,0,1,0,1,0,1],
["Gb",-1,0,-1,0,-1,0,0,-1,0,-1,0,0],
["Db",-1,0,-1,0,0,-1,0,-1,0,-1,0,0],
["Ab",-1,0,-1,0,0,-1,0,-1,0,0,-1,0],
["Eb",-1,0,0,-1,0,-1,0,-1,0,0,-1,0],
["Bb",-1,0,0,-1,0,-1,0,0,0,-1,0],
["F",0,0,0,-1,0,-1,0,0,-1,0,-1,0],
["F#",1,0,1,0,1,0,0,1,0,1,0,0],
["C#",1,0,1,0,0,1,0,1,0,1,0,0],
];

var altconvert = [
["Gb7alt","C7"],
["Db7alt","G7"],
["Ab7alt","D7"],
["Eb7alt","A7"],
["Bb7alt","E7"],
["F7alt","B7"],
["C7alt","Gb7"],
["G7alt","Db7"],
["D7alt","Ab7"],
["A7alt","Eb7"],
["E7alt","Bb7"],
["B7alt","F7"],
["F#7alt","C7"],
["C#7alt","G7"],
["G#7alt","D7"],
["D#7alt","A7"],
["A#7alt","E7"],
];
        var tpc2note = ["cbb","gbb","dbb","abb","ebb","bbb","fb","cb","gb","db","ab","eb","bb","f","c","g","d","a","e","b","f#","c#","g#","d#","a#","e#","b#","f##","c##","g##","d##","a##","e##","b##"];
		var maxembel = 20;
		var mingap = 480;
		var cursor = curScore.newCursor()
		if (typeof curScore === 'undefined') {
			// //console.log("Please open a jazz lead sheet before calling execution of this function.");
			Qt.quit();
		}
        // collect the melody notes and their segment.tick
		var melnotes = new Array(512); 
		// melnotes
		// O: notename
		// 1: tick
		for(var i=0; i<512; i++) melnotes[i]=new Array();
		var mcount = 0;
        cursor.rewind(0); // beginning of score

        while (cursor.segment) {
            if (cursor.element && cursor.element.type === Element.CHORD) {
                var notes = cursor.element.notes;
                //var name;
				// only register top note
				i = notes.length - 1;
                melnotes[mcount][0] = tpc2note[notes[i].tpc];
				melnotes[mcount][1] = cursor.segment.tick;
				mcount++;
            }// end if CHORD 
            cursor.next();
            
        } // end while segment

		//for (var c = 0; c < mcount; c++){
		//	// //console.log("note: ", melnotes[c][0] + " " + melnotes[c][1]);
		//}

        cursor.rewind(0); // beginning of score    
		cursor.voice = 1;

		//0: chord pair:"chordtype1 distance chordtype2"
		//1: distance to embed chord in half notes: from -5 (eg C-G) to 6 (eg C- F#)
		//2: embed chord type
		//3: acceptable melnotes relative to embed chords (i iib ii iiib iii iv etc...)
		// Note: bass of slash chords in absolute steps from root, e.g. D7/A -> 7/7 not 7/-5
		// Note: 7 and m7b5 voicings are identical: e.g. d g ab c = Dm7b5 = Bb7
		// Sometimes both of these will appear in the same embellish stack
		// This is because there is also another regular voicing of m7b5 = 7 1 3 b5, e.g. am7b5 = g a c eb
		// For slash chords use literal chord 1357, not voicings
		// 7#11 chords: e.g. F7#11 = G7b13 = f a b eb or inversions
		// or F9#11: a b eb g (not explored yet)
		// sorted for most prevalent first ie lowest on the score
        // ptype sorted lo to hi
    var embel = [
["7 0 7b9",-5,"m7","001001000100"],
["7 0 7sus",4,"dim","000001000000"],
["7 0 7sus",-5,"m7","100001000000"],
["7 0 dim",-1,"7","000100000000"],
["7 0 dim",4,"m7b5","000000001000"],
["7 0 m7",2,"7b13","000000000010"],
["7 0 m7",-3,"7b13","000000000010"],
["7 0 m7",6,"7b9","010000000000"],
["7 0 m7",3,"m7","000000010000"],
["7 0 m7",0,"m7b5","001000000010"],
["7 0 m7",-4,"7","000010100000"],
["7 0 m7",-5,"7b13","101000001010"],
["7 0 m7",3,"7","101000110100"],
["7 0 m7",1,"dim","111111101101"],
["7 0 m7",1,"7","011010101101"],
["7 0 m7",-5,"m7b5","101001011010"],
["7 0 m7",6,"7","111010111111"],
["7 0 m7",0,"7b13","111111111010"],
["7 0 m7b5",3,"7b13","000000100000"],
["7 0 m7b5",1,"7b13","000000001000"],
["7 0 m7b5",6,"7","000000100000"],
["7 0 m7b5",1,"dim","000000101000"],
["7 0 m7b5",0,"7b13","101000000000"],
["7 0 m7b5",-5,"m7b5","101001000000"],
["7 1 7",-5,"7b13","000010000000"],
["7 1 7",3,"7","000010000000"],
["7 1 7",5,"7","001000000100"],
["7 1 7",-4,"7","001010100000"],
["7 1 7",-5,"m7","101101011000"],
["7 -1 7",-1,"7b13","001000000000"],
["7 -1 7",-4,"7b13","000010000000"],
["7 -1 7",5,"7b9","010000000000"],
["7 -1 7",-5,"7b9","010000000000"],
["7 -1 7",6,"7b9","010000000000"],
["7 -1 7",1,"7b13","000000010010"],
["7 -1 7",6,"7sus","000001000000"],
["7 -1 7",-5,"7","000010000100"],
["7 -1 7",-1,"m7b5","000100000000"],
["7 -1 7",-1,"7sus","101001011101"],
["7 -1 7",0,"7b13","111000101010"],
["7 -1 7",6,"7","1010010110112"],
["7 -1 7",5,"7","11110111110010"],
["7 1 7#11",-5,"m7","000001000000"],
["7 1 7alt",-5,"7b13","000010000000"],
["7 1 7alt",3,"7","000010000000"],
["7 1 7alt",5,"7","001000000000"],
["7 1 7alt",-5,"m7","100010000000"],
["7 -1 7alt",-5,"7","000000000100"],
["7 -1 7alt",-5,"7b9","010000000000"],
["7 -1 7alt",6,"7b9","010000000000"],
["7 -1 7alt",-1,"7sus","000001001000"],
["7 -1 7alt",-1,"m7b5","000100000000"],
["7 -1 7alt",6,"7","010010100010"],
["7 -1 7alt",0,"7b13","101000001010"],
["7 -1 7alt",5,"7","001010010100"],
["7 -1 7b9",6,"m7","100000000000"],
["7 -1 7b9",6,"m7b5","000101000010"],
["7 -1 7sus",6,"m7","000000000010"],
["7 1 dim",0,"m7","000000010000"],
["7 1 m7",1,"7b13","000000001000"],
["7 1 m7",-1,"7","000000000010"],
["7 1 m7",3,"7","000010000000"],
["7 1 m7",-1,"7b13","000000001010"],
["7 1 m7",4,"m7","000101000000"],
["7 -1 m7",2,"m7b5","000100000000"],
["7 -1 m7",1,"7b13","100000000000"],
["7 -1 m7",3,"7b9","010000000000"],
["7 -1 m7",6,"7b9","010000000000"],
["7 -1 m7",3,"7b13","000000001010"],
["7 -1 m7",5,"7b13","000010001000"],
["7 -1 m7",0,"7b13","111010100100"],
["7 -1 m7",3,"7","010010110100"],
["7 -1 m7",2,"7b13","101010010010"],
["7 -1 m7",-2,"dim","011110101101"],
["7 -1 m7",4,"m7b5","101101000100"],
["7 -1 m7",6,"7","101010111010"],
["7 -1 m7",5,"7","101010111001"],
["7 1 m7b5",5,"7b13","000000001000"],
["7 1 m7b5",5,"7","001000010000"],
["7 1 m7b5",4,"m7","100000001000"],
["7 -1 m7b5",-1,"7b13","001000000000"],
["7 -1 m7b5",2,"7","000010000000"],
["7 -1 m7b5",3,"dim","000000011000"],
["7 -1 m7b5",-1,"7","001000000000"],
["7 -1 m7b5",6,"m7b5","000100000000"],
["7 -1 m7b5",4,"7b13","100000100110"],
["7 -1 m7b5",5,"7","011010101000"],
["7 -1 m7b5",-2,"7","100110110100"],
["7 1 Maj",-2,"7b13","000000000010"],
["7 1 Maj",3,"7b13","000010000000"],
["7 1 Maj",-1,"7","000000000100"],
["7 1 Maj",2,"7b13","000000000010"],
["7 1 Maj",5,"7b13","001000000000"],
["7 1 Maj",2,"7","001000100000"],
["7 1 Maj",6,"7","000010000010"],
["7 1 Maj",-5,"m7","100000000000"],
["7 1 Maj",0,"7b13","100000001000"],
["7 1 Maj",-4,"7","100010100000"],
["7 -1 Maj",-5,"7b13","000010000000"],
["7 -1 Maj",6,"7sus","000001000000"],
["7 -1 Maj",2,"7","001000001000"],
["7 -1 Maj",-2,"7b9","010000000000"],
["7 -1 Maj",1,"m7b5","000100000100"],
["7 -1 Maj",-2,"m7b5","100000100000"],
["7 -1 Maj",5,"7b9","010000000000"],
["7 -1 Maj",-2,"dim","100000101000"],
["7 -1 Maj",-4,"m7b5","000100000000"],
["7 -1 Maj",4,"m7b5","001000000000"],
["7 -1 Maj",-5,"m7b5","001100000000"],
["7 -1 Maj",6,"m7b5","000010001010"],
["7 -1 Maj",-2,"7","110000100100"],
["7 -1 Maj",4,"7b13","101010010010"],
["7 -1 Maj",4,"7","001000110000"],
["7 -1 Maj",-1,"7b13","011010001010"],
["7 -1 Maj",-3,"7","101110000010"],
["7 -1 Maj",5,"7","001010100100"],
["7 -1 Maj",2,"7b13","101010001011"],
["7 -1 Maj",6,"7","101111110110"],
["7 -1 Maj",0,"7b13","001010111011"],
["7 2 7",3,"7b13","000000100000"],
["7 2 7",6,"7b13","010000000000"],
["7 2 7",-5,"7sus","000001000000"],
["7 2 7",-5,"7","000001000100"],
["7 2 7",6,"7","000000010000"],
["7 2 7",1,"7","110000100000"],
["7 2 7",5,"7b13","001010001000"],
["7 2 7",3,"7","000000110111"],
["7 -2 7",1,"7b13","000000001000"],
["7 -2 7",-1,"7","000000010000"],
["7 -2 7",-4,"7b9","010000000000"],
["7 -2 7",2,"7b13","001000000000"],
["7 -2 7",6,"7b9","010000000000"],
["7 -2 7",-1,"m7b5","000100000000"],
["7 -2 7",3,"7","000010000000"],
["7 -2 7",6,"7b13","010000001000"],
["7 -2 7",-4,"7","010010000001"],
["7 -2 7",-5,"7","101000000000"],
["7 -2 7",6,"7","011000100000"],
["7 -2 7",5,"7","011010010101"],
["7 2 m7",0,"7b13","100000000000"],
["7 2 m7",-1,"7b13","000000001000"],
["7 2 m7",3,"7b13","010000000000"],
["7 2 m7",-4,"7b13","000010000000"],
["7 2 m7",-4,"7","000010000000"],
["7 2 m7",3,"7alt","000000000100"],
["7 2 m7",0,"7b9","000000010000"],
["7 2 m7",3,"7b9","010000000000"],
["7 2 m7",-5,"m7b5","000001000000"],
["7 2 m7",5,"7","000000010000"],
["7 2 m7",6,"7","000000110000"],
["7 2 m7",-3,"7","000000000010"],
["7 2 m7",-2,"mMaj7","000000000100"],
["7 2 m7",-2,"dim","000100100100"],
["7 2 m7",3,"7","000010100100"],
["7 2 m7",-2,"7","001000100100"],
["7 -2 m7",-4,"7","000000100000"],
["7 -2 m7",-4,"7b9","010000000000"],
["7 -2 m7",0,"dim","100000000100"],
["7 -2 m7",5,"7","000010010100"],
["7 2 m7b5",1,"7","000000000001"],
["7 2 Maj",0,"7b13","001000000000"],
["7 2 Maj",5,"7b13","000010000000"],
["7 2 Maj",6,"m7b5","000100000000"],
["7 2 Maj",-4,"7b9","010000000000"],
["7 2 Maj",-4,"7","010000000100"],
["7 2 Maj",-3,"7b13","000000001010"],
["7 2 Maj",1,"7","000010000100"],
["7 2 Maj",-3,"7","000000011100"],
["7 2 Maj",1,"dim","010111100000"],
["7 2 Maj",2,"7b13","101111010000"],
["7 2 Maj",3,"7","011010110111"],
["7 -2 Maj",-3,"7b13","000000001000"],
["7 -2 Maj",5,"7susb9","000001000000"],
["7 -2 Maj",0,"m7b5","000000000001"],
["7 -2 Maj",3,"mMaj7","000000010000"],
["7 -2 Maj",-1,"7b13","000000101000"],
["7 -2 Maj",4,"dim","000100100000"],
["7 3 7",0,"7b13","100000000000"],
["7 3 7",6,"7b9","010000000000"],
["7 3 7",5,"7sus","000001000000"],
["7 3 7",-2,"mMaj7","000000000001"],
["7 3 7",6,"7","001010100000"],
["7 3 7",-2,"m7","100000000100"],
["7 3 7",-3,"7","110000000000"],
["7 3 7",-5,"m7","100100000000"],
["7 3 7",5,"7","101011010000"],
["7 3 7",-2,"7","101000010111"],
["7 -3 7",-5,"7","001000000000"],
["7 -3 7",3,"7b9","010000000000"],
["7 -3 7",-3,"m7b5","000000000001"],
["7 -3 7",0,"7b13","101000000000"],
["7 -3 7",-4,"7","100010000000"],
["7 -3 7",6,"7","000000100010"],
["7 -3 7",4,"m7b5","100001000000"],
["7 -3 7",5,"7","001000010100"],
["7 -3 7",3,"7","001011100100"],
["7 -3 7",-2,"7","001010110111"],
["7 3 7#11",4,"m7b5","100000000000"],
["7 3 7#11",5,"7","001000000000"],
["7 3 7alt",-3,"7b9","010000000000"],
["7 3 7alt",5,"7sus","000001000000"],
["7 3 7alt",6,"7b13","000000100010"],
["7 3 7alt",-2,"m7","100000000100"],
["7 3 7alt",-5,"m7","100100000000"],
["7 3 7alt",5,"7","001000010001"],
["7 3 7alt",6,"7","001010100010"],
["7 3 7alt",-2,"7","101000100110"],
["7 -3 dim",-5,"m7","000100000000"],
["7 3 m7",1,"m7","000001000000"],
["7 3 m7",6,"7","100010000000"],
["7 3 m7",0,"7b13","100000100010"],
["7 3 m7",1,"7","000000100000"],
["7 3 m7",5,"7","001010000000"],
["7 3 m7",-2,"dim","101000000100"],
["7 -3 m7",3,"7b9","010000000000"],
["7 -3 m7",6,"7b9","010000000000"],
["7 -3 m7",2,"m7b5","000000000010"],
["7 -3 m7",0,"7b13","101011000010"],
["7 -3 m7",6,"7","011010100010"],
["7 -3 m7",-2,"dim","101000110101"],
["7 -3 m7",-2,"m7","101000010001"],
["7 -3 m7",-2,"mMaj7","101000010101"],
["7 -3 m7",3,"7","011010110101"],
["7 -3 m7",-2,"7","101010110110"],
["7 -3 m7b5",6,"7b9","010000000000"],
["7 -3 m7b5",-2,"m7b5","000110100000"],
["7 -3 m7b5",0,"7b13","101000000000"],
["7 -3 m7b5",-3,"m7","000100011000"],
["7 -3 m7b5",3,"7","000000100110"],
["7 -3 m7b5",6,"7","111010100111"],
["7 -3 m7b5",-2,"7","101000110111"],
["7 3 Maj",-1,"7","000000000010"],
["7 3 mMaj7",0,"m7","101000000010"],
["7 4 7",5,"7b13","000000001000"],
["7 4 7",-5,"7","000000000001"],
["7 4 7",-1,"7","001000010000"],
["7 4 7",-3,"7","100000000100"],
["7 4 7",5,"7","010010000000"],
["7 -4 7",3,"7b13","000010000000"],
["7 -4 7",1,"7","000000100000"],
["7 -4 7",-2,"7","000000000100"],
["7 -4 7",5,"7","001000000000"],
["7 -4 7",3,"7","000010000100"],
["7 -4 7alt",1,"7","000000100000"],
["7 -4 7alt",-2,"7","000000000100"],
["7 -4 7alt",5,"7","001000000000"],
["7 -4 7alt",0,"m7b5","000000100000"],
["7 -4 7alt",3,"7","000010000100"],
["7 4 7sus",5,"7","000000000001"],
["7 4 7sus",-1,"m7b5","000000000010"],
["7 4 m7",-1,"7b13","000000100000"],
["7 4 m7",5,"7b9","010000000000"],
["7 4 m7",-2,"7sus","000001000000"],
["7 4 m7",-2,"7b13","000000001000"],
["7 4 m7",6,"7b13","100000000001"],
["7 4 m7",6,"7","100000000100"],
["7 4 m7",3,"7","100000010100"],
["7 4 m7",-2,"7","101000110000"],
["7 4 m7",5,"7","011010010000"],
["7 4 m7",2,"7b13","001010001010"],
["7 -4 m7",3,"7","000010000000"],
["7 -4 m7",2,"7","100010000000"],
["7 -4 m7",-5,"7","000000010001"],
["7 -4 m7",6,"7","001010100010"],
["7 4 m7b5",5,"7b13","000010000000"],
["7 4 m7b5",-2,"7","000000100000"],
["7 4 m7b5",0,"7b9","000000000100"],
["7 4 m7b5",-2,"dim","000000000100"],
["7 4 m7b5",-5,"m7","101001000000"],
["7 4 m7b5",3,"7","000010110010"],
["7 -4 m7b5",-1,"7","010000010000"],
["7 4 Maj",-2,"7b13","000000001000"],
["7 4 Maj",-1,"7","000000010000"],
["7 4 Maj",-2,"7","000000010000"],
["7 4 Maj",5,"7b9","010000000000"],
["7 4 Maj",-5,"m7b5","000001000000"],
["7 4 Maj",3,"7","001000000100"],
["7 4 Maj",5,"7","100000010000"],
["7 -4 Maj",3,"7","000000000100"],
["7 -4 Maj",1,"dim","000100000001"],
["7 -4 Maj",6,"7","000000100010"],
["7 -5 7",2,"7b13","000000001000"],
["7 -5 7",-4,"7b9","010000000000"],
["7 -5 7",5,"7b9","010000000000"],
["7 -5 7",5,"7sus","000001000000"],
["7 -5 7",0,"7b13","000000000010"],
["7 -5 7",2,"7","100000000000"],
["7 -5 7",6,"7","000010000000"],
["7 -5 7",-2,"7","100010000000"],
["7 -5 7",3,"7","000000110000"],
["7 -5 7",5,"7","000011000100"],
["7 -5 7",-4,"7","011000100000"],
["7 5 7#11",-4,"7b13","000000000010"],
["7 5 7#11",-1,"7","000000010000"],
["7 5 7sus",-1,"7b13","000000001000"],
["7 5 7sus",6,"7b9","010000000000"],
["7 5 7sus",-1,"7","010001000000"],
["7 5 7sus",-1,"7b9","010000000000"],
["7 5 7sus",-2,"dim","101000100001"],
["7 5 7sus",6,"7","010010100010"],
["7 5 7sus",0,"m7","100010010010"],
["7 5 dim",6,"7b13","000000001010"],
["7 5 dim",0,"7b9","100000010000"],
["7 5 dim",6,"7","010010100000"],
["7 -5 m7",-2,"7b13","000010000000"],
["7 -5 m7",-2,"m7b5","000000100000"],
["7 -5 m7",6,"7b9","010000000000"],
["7 -5 m7",-1,"7b9","010000000000"],
["7 -5 m7",-1,"7b13","000000001000"],
["7 -5 m7",-5,"7b13","001000000000"],
["7 -5 m7",3,"7","000000100000"],
["7 -5 m7",1,"7b9","010000000000"],
["7 -5 m7",-2,"7","000000010000"],
["7 -5 m7",-4,"7b13","000010000000"],
["7 -5 m7",-1,"7","001000100000"],
["7 -5 m7",-4,"7","000010100000"],
["7 -5 m7",6,"7","000010101010"],
["7 -5 m7",0,"7b13","101010010000"],
["7 -5 m7",1,"7","010010100101"],
["7 5 m7b5",0,"7b13","000000000001"],
["7 5 m7b5",-1,"7","000000000100"],
["7 5 m7b5",6,"7","001010100000"],
["7 -5 m7b5",1,"7","000000100100"],
["7 5 Maj",6,"7b13","001000001000"],
["7 5 Maj",6,"7","000000001000"],
["7 5 Maj",5,"7b13","0000111001000"],
["7 -5 Maj",3,"7","000000000100"],
["7 -5 Maj",-4,"7b9","010000000000"],
["7 -5 Maj",0,"7b13","000010000000"],
["7 -5 Maj",-4,"7b13","000000001000"],
["7 -5 Maj",-5,"m7b5","100000000000"],
["7 -5 Maj",1,"dim","010101101100"],
["7 -5 Maj",2,"7","101000010000"],
["7 -5 Maj",5,"7","001010000100"],
["7 -5 Maj",-4,"7","011010100001"],
["7 5 mMaj7",6,"7","001000000000"],
["7 5 mMaj7",1,"7b9","010000000000"],
["7 5 mMaj7",1,"7","000000100000"],
["7 6 7#11",-5,"m7","000001010010"],
["7 6 dim",5,"7","000000000100"],
["7 6 dim",-5,"m7","100001000100"],
["7 6 m7",-5,"7b9","010000000000"],
["7 6 m7",5,"7b9","010000000000"],
["7 6 m7",6,"7b9","010000000000"],
["7 6 m7",5,"7sus","000001000000"],
["7 6 m7",3,"7b9","010000000000"],
["7 6 m7",6,"7sus","000001000000"],
["7 6 m7",1,"7b13","010000001000"],
["7 6 m7",3,"7","010000100100"],
["7 6 m7",-5,"7","011000010000"],
["7 6 m7",-1,"7b13","000010001010"],
["7 6 m7",-5,"m7b5","100001000100"],
["7 6 m7",5,"7","101011000100"],
["7 6 m7",4,"dim","101111001011"],
["7 6 m7",4,"m7b5","101101001010"],
["7 6 m7",0,"7b13","111111111000"],
["7 6 m7",6,"7","101011110111"],
["7 6 m7b5",-2,"7","000010000000"],
["7 6 m7b5",4,"m7b5","001000000000"],
["7 6 m7b5",-1,"7","000000000010"],
["7 6 m7b5",3,"7","000010100101"],
["7 6 m7b5",5,"7","001010010101"],
["7 6 Maj",1,"7","000000000001"],
["7 6 Maj",-1,"7","010000000000"],
["7 6 Maj",5,"7b9","010000000000"],
["7#11 0 7",2,"7","000010000000"],
["7#11 -1 7",1,"7b13","000000001000"],
["7#11 -1 7",6,"7sus","100000000000"],
["7#11 -1 7",5,"7","001010000000"],
["7#11 -1 7",-1,"7sus","000000010100"],
["7#11 -1 m7",6,"7","100000000010"],
["7#11 1 Maj",0,"7b13","000000001000"],
["7#11 -1 Maj",6,"7","100000000000"],
["7#11 2 Maj",-1,"7b13","000000001000"],
["7#11 2 Maj",6,"7b13","001000000000"],
["7#11 2 Maj",3,"7sus","000001000000"],
["7#11 2 Maj",0,"dim","000000001000"],
["7#11 2 Maj",-2,"m7b5","000000000010"],
["7#11 2 Maj",-2,"7","000010000100"],
["7#11 2 Maj",3,"7","000010000001"],
["7#11 2 Maj",-3,"m7b5","000000010000"],
["7#11 2 Maj",-3,"7","000000010000"],
["7#11 2 Maj",1,"dim","000101100000"],
["7#11 3 m7",5,"7","001000000000"],
["7#11 -3 m7",-1,"7","000000010000"],
["7#11 4 7",-1,"7","000000010000"],
["7#11 4 7",-5,"mMaj7","000100000000"],
["7alt 0 m7",1,"dim","000001000000"],
["7alt 0 m7",0,"7b13","000000100000"],
["7alt 0 m7",6,"7","100000000000"],
["7alt -1 7",1,"7b13","000000010000"],
["7alt -1 7",-4,"7b13","000010000000"],
["7alt -1 7",-5,"7b9","010000000000"],
["7alt -1 7",4,"7sus","000001000000"],
["7alt -1 7",-1,"7b13","001000000000"],
["7alt -1 7",6,"7b9","010000000000"],
["7alt -1 7",6,"7sus","000001000000"],
["7alt -1 7",5,"7b9","010000000000"],
["7alt -1 7",-1,"m7b5","000100000000"],
["7alt -1 7",-5,"7","010010000100"],
["7alt -1 7",0,"7b13","111000111010"],
["7alt -1 7",-1,"7sus","101001011110"],
["7alt -1 7",5,"7","111010110110"],
["7alt -1 7",6,"7","110010110111"],
["7alt -1 7sus",6,"m7","101000000010"],
["7alt 1 m7",1,"7b13","000000001000"],
["7alt 1 m7",-1,"7b13","000000000010"],
["7alt 1 m7",-1,"7","000000000010"],
["7alt 1 m7",4,"m7b5","000001000000"],
["7alt -1 m7",-3,"7","001000000000"],
["7alt -1 m7",3,"7b9","010000000000"],
["7alt -1 m7",2,"m7b5","000100000000"],
["7alt -1 m7",6,"7b9","010000000000"],
["7alt -1 m7",1,"7b13","101000000000"],
["7alt -1 m7",3,"7b13","100000001010"],
["7alt -1 m7",5,"7b13","000010001000"],
["7alt -1 m7",2,"7b13","101010010010"],
["7alt -1 m7",0,"7b13","011010100100"],
["7alt -1 m7",-2,"dim","010111101101"],
["7alt -1 m7",4,"m7b5","101101000101"],
["7alt -1 m7",6,"7","101010110110"],
["7alt -1 m7",3,"7","010110111101"],
["7alt -1 m7",5,"7","101110111011"],
["7alt 1 m7b5",5,"7b9","010000000000"],
["7alt 1 m7b5",5,"7","000010100000"],
["7alt 1 m7b5",4,"m7","100000000000"],
["7alt -1 m7b5",0,"7b13","000010000000"],
["7alt -1 m7b5",-1,"7b13","001000000000"],
["7alt -1 m7b5",-1,"7","001000000000"],
["7alt -1 m7b5",0,"dim","100000000000"],
["7alt -1 m7b5",5,"7b9","010000000000"],
["7alt -1 m7b5",5,"7","011000010000"],
["7alt -1 m7b5",-2,"7","001010100100"],
["7alt -1 m7b5",4,"7b13","101000001010"],
["7alt -1 Maj",-2,"7b9","010000000000"],
["7alt -1 Maj",1,"7b13","000000000010"],
["7alt -1 Maj",5,"7b9","010000000000"],
["7alt -1 Maj",-4,"m7b5","000100000000"],
["7alt -1 Maj",4,"m7b5","001000000000"],
["7alt -1 Maj",-5,"dim","000000000101"],
["7alt -1 Maj",1,"m7b5","000100000000"],
["7alt -1 Maj",-2,"m7b5","000000100000"],
["7alt -1 Maj",-5,"m7b5","000100000000"],
["7alt -1 Maj",2,"7","001000010000"],
["7alt -1 Maj",6,"m7b5","000100001010"],
["7alt -1 Maj",4,"7","001000110000"],
["7alt -1 Maj",-2,"7","001000100111"],
["7alt -1 Maj",5,"7","011010100100"],
["7alt -1 Maj",-1,"7b13","101010101010"],
["7alt -1 Maj",-3,"7","101010000010"],
["7alt -1 Maj",4,"7b13","101010110010"],
["7alt -1 Maj",2,"7b13","101010101001"],
["7alt -1 Maj",0,"7b13","101010111111"],
["7alt -1 Maj",6,"7","101011110111"],
["7alt -1 mMaj7",5,"7","010000000000"],
["7alt -1 mMaj7",5,"7b9","010000000000"],
["7alt 2 7",5,"7b13","010000000000"],
["7alt 2 7",6,"7","100000000000"],
["7alt 2 7",1,"7sus","000001000000"],
["7alt 2 7",-3,"m7","000000000100"],
["7alt 2 Maj",-4,"7b9","010000000000"],
["7alt 2 Maj",1,"dim","000001100000"],
["7alt 2 Maj",-3,"7b13","000000000011"],
["7alt 2 Maj",3,"7","000010100000"],
["7alt 2 Maj",-3,"7","000000000100"],
["7alt 2 Maj",2,"7b13","000010010000"],
["7alt -3 7",-2,"7","000000000100"],
["7alt -3 7",3,"7","000010000000"],
["7alt -3 7",5,"7","001000000000"],
["7alt 3 7alt",-2,"7","000000000100"],
["7alt 3 7alt",5,"7","001000000000"],
["7alt 3 7alt",-2,"m7","000000000100"],
["7alt 3 7alt",-5,"m7","100000000000"],
["7alt -3 7alt",6,"7b13","000000001000"],
["7alt -3 7alt",-2,"7","000010000000"],
["7alt -3 7alt",5,"7","000000000100"],
["7alt 3 m7",1,"7","000000100000"],
["7alt 3 m7",1,"dim","000000100100"],
["7alt 3 m7",0,"7b13","000000000110"],
["7alt 3 m7",5,"7","001010000000"],
["7alt 3 m7",6,"7","000010000000"],
["7alt -3 m7",-2,"dim","000000001000"],
["7alt -3 m7",0,"7b13","000000100000"],
["7alt -3 m7",6,"7","100000000000"],
["7alt -3 m7",-2,"7b13","000000001000"],
["7alt 3 m7b5",6,"7","000010000000"],
["7alt 3 m7b5",6,"7b13","100100000000"],
["7alt 4 7",5,"7b13","000000001000"],
["7alt 4 7",-1,"7","001000000000"],
["7alt 4 7sus",5,"7","000010000000"],
["7alt 4 7sus",6,"7","100000000000"],
["7alt 4 7sus",-1,"m7b5","001000000010"],
["7alt 4 m7",-1,"7b13","001000000000"],
["7alt 4 m7",4,"m7b5","000000000100"],
["7alt 4 m7b5",-2,"7","000000100000"],
["7alt -4 m7b5",4,"7","000000000100"],
["7alt 4 Maj",1,"7b13","000000100000"],
["7alt 4 Maj",-1,"7b13","000000001000"],
["7alt 4 Maj",-2,"7","000000000100"],
["7alt 4 Maj",3,"7","000010000000"],
["7alt 4 Maj",5,"7","001000000000"],
["7alt -5 7",5,"7","010000000000"],
["7alt -5 7",5,"7b9","010000000000"],
["7alt -5 m7",0,"7b13","000010000000"],
["7alt -5 m7",6,"7","000000000010"],
["7alt 5 m7b5",6,"7","100000000000"],
["7alt 5 Maj",6,"7","000000001000"],
["7alt 5 Maj",5,"7b13","000010000010"],
["7alt -5 Maj",-4,"7","001000000000"],
["7alt -5 Maj",2,"7b13","001010000000"],
["7alt -5 Maj",2,"7","001000000000"],
["7alt -5 Maj",1,"dim","000101000100"],
["7alt -5 Maj",0,"7b13","000010000010"],
["7alt 6 7b9",-1,"7","000000100000"],
["7alt 6 m7",3,"7b9","010000000000"],
["7alt 6 m7",5,"7b9","010000000000"],
["7alt 6 m7",-1,"7sus","000000010000"],
["7alt 6 m7",-5,"7","010000010000"],
["7alt 6 m7",6,"7sus","100001000000"],
["7alt 6 m7",-5,"m7b5","101000000000"],
["7alt 6 m7",1,"7b13","000000001000"],
["7alt 6 m7",-1,"7b13","000000001010"],
["7alt 6 m7",0,"7b13","011000111000"],
["7alt 6 m7",5,"7","011010100100"],
["7alt 6 m7",4,"dim","101111000010"],
["7alt 6 m7",4,"m7b5","101101000010"],
["7alt 6 m7",6,"7","101001010010"],
["7alt 6 m7b5",6,"7","100000000000"],
["7alt 6 m7b5",4,"m7b5","001000000000"],
["7alt 6 m7b5",-1,"7","000000000010"],
["7alt 6 m7b5",3,"7","000010100000"],
["7alt 6 m7b5",5,"7","001010000000"],
["7b13 5 Maj",0,"7","000001000000"],
["7b9 5 m7",2,"7b13","000000000010"],
["7b9 5 m7",0,"7sus","000001000000"],
["7b9 5 m7",6,"7b9","010000000000"],
["7b9 5 m7",4,"dim","100101101110"],
["7b9 5 m7",0,"7","111000010110"],
["7b9 5 m7",6,"7","0111110111004"],
["7b9 6 m7b5",3,"7","000000000010"],
["7sus 0 7",-2,"dim","001000000000"],
["7sus 0 7",3,"dim","001010100101"],
["7sus 2 m7",0,"7sus","010000000000"],
["dim 0 Maj",5,"7","000000010000"],
["dim 1 7",1,"7sus","000000000100"],
["dim 1 m7",2,"7b13","000000001000"],
["dim 1 m7",2,"7b9","010000000000"],
["dim 1 m7",-4,"7","101010110100"],
["dim 1 m7",2,"7","1110110110000"],
["dim -1 m7",2,"7b13","000010000000"],
["dim -1 m7",-2,"7","000000001000"],
["dim -1 m7",5,"7alt","000010000000"],
["dim -1 m7",0,"7","000000100000"],
["dim -1 m7",-1,"7","000010010110"],
["dim 2 m7",1,"7b13","000000001000"],
["dim 2 m7",2,"7","000000010000"],
["dim 2 m7",3,"7","000000100000"],
["dim -2 m7",5,"7","000000000100"],
["dim 3 Maj",4,"7b13","000000000001"],
["dim 3 Maj",-2,"7","000010000000"],
["dim -3 Maj",4,"7b13","001000000000"],
["dim -3 Maj",4,"7","001001000000"],
["dim 4 7",4,"7sus","001000000000"],
["dim 4 m7",5,"7","000000010000"],
["dim -4 m7",-5,"7b13","000000001000"],
["dim -4 m7",-3,"7","000000100000"],
["dim -4 m7",-4,"7","000000010010"],
["dim 5 m7",5,"7","000000000100"],
["dim -5 m7",2,"7","000010000000"],
["dim -5 m7",-4,"7","000010000000"],
["dim 6 Maj",5,"7","000000000100"],
["m7 0 7",1,"7b13","000000001000"],
["m7 0 7",1,"7","001011000001"],
["m7 0 7",5,"7","110010010110"],
["m7 0 dim",5,"7b13","001000000000"],
["m7 0 dim",1,"7b9","010000000000"],
["m7 0 m7b5",-1,"7","000000100000"],
["m7 0 m7b5",5,"7","100010010000"],
["m7 0 m7b5",3,"m7","101000100100"],
["m7 0 m7b5",6,"7","000010100111"],
["m7 0 m7b5",3,"mMaj7","101000100101"],
["m7 1 7",2,"m7","000100000000"],
["m7 1 7",-4,"mMaj7","000000000001"],
["m7 1 7",-4,"7","001010010100"],
["m7 1 7",2,"m7b5","010101000010"],
["m7 1 7",5,"7","101011010010"],
["m7 -1 7",5,"7b13","000010000000"],
["m7 -1 7",5,"7sus","000001000000"],
["m7 -1 7",6,"7b13","000000001000"],
["m7 -1 7",6,"7b9","010000000000"],
["m7 -1 7",1,"7","000000100100"],
["m7 -1 7",1,"7b13","000110000000"],
["m7 -1 7",5,"m7b5","000001000001"],
["m7 -1 7",2,"7b13","010000101010"],
["m7 -1 7",5,"7b9","100110010111"],
["m7 -1 7",6,"7","011010100111"],
["m7 -1 7",0,"m7b5","100111001010"],
["m7 -1 7",5,"7","101011010111"],
["m7 -1 7#11",6,"7","000000000001"],
["m7 -1 7#11",5,"7","101001000010"],
["m7 1 7alt",5,"7sus","000001000000"],
["m7 1 7alt",2,"m7","000000001000"],
["m7 1 7alt",-4,"mMaj7","000000000101"],
["m7 1 7alt",2,"m7b5","010001001010"],
["m7 1 7alt",-4,"7","001010110111"],
["m7 1 7alt",5,"7","111001010110"],
["m7 -1 7alt",5,"7sus","000001000000"],
["m7 -1 7alt",3,"7b13","000010000000"],
["m7 -1 7alt",5,"7b13","000010000000"],
["m7 -1 7alt",6,"7b9","010000000000"],
["m7 -1 7alt",6,"7b13","000000101000"],
["m7 -1 7alt",1,"7b13","000010000000"],
["m7 -1 7alt",1,"7","100000100100"],
["m7 -1 7alt",5,"m7b5","100001000000"],
["m7 -1 7alt",2,"7b13","010010001010"],
["m7 -1 7alt",5,"7b9","110111010110"],
["m7 -1 7alt",0,"m7b5","1101101001010"],
["m7 -1 7alt",6,"7","111010110101"],
["m7 -1 7alt",5,"7","101111011110"],
["m7 -1 7b9",5,"m7","100000000000"],
["m7 1 dim",5,"7sus","000001000000"],
["m7 1 dim",5,"7","001001010000"],
["m7 -1 dim",5,"7#9","000100000000"],
["m7 -1 dim",5,"7sus","000001000000"],
["m7 -1 dim",1,"7b13","000010000000"],
["m7 -1 dim",5,"7b9","000000000010"],
["m7 -1 dim",3,"dim","000000010000"],
["m7 -1 dim",-1,"7b13","000000001000"],
["m7 -1 dim",6,"7","000010000000"],
["m7 -1 dim",-1,"7","000010100100"],
["m7 -1 dim",5,"7","101001000010"],
["m7 1 m7",-4,"7","000010100000"],
["m7 1 m7",-1,"7","000010000000"],
["m7 1 m7",5,"7","000000010110"],
["m7 -1 m7",5,"7b13","000000001000"],
["m7 -1 m7",1,"7","000000000100"],
["m7 -1 m7",5,"7","101010010110"],
["m7 1 Maj",-1,"7b13","000000001000"],
["m7 1 Maj",-4,"7","000000000100"],
["m7 1 Maj",-1,"7b9","010000000000"],
["m7 1 Maj",5,"m7","000000000010"],
["m7 1 Maj",5,"7","101000000010"],
["m7 1 Maj",-1,"7","011010000000"],
["m7 -1 Maj",-1,"7","000000000001"],
["m7 -1 Maj",-4,"7","000000010000"],
["m7 -1 Maj",5,"7sus","000001000000"],
["m7 -1 Maj",5,"m7","000000000010"],
["m7 -1 Maj",0,"m7b5","000000000010"],
["m7 -1 Maj",5,"7","000001000010"],
["m7 2 7",-1,"7","000000100000"],
["m7 2 7",5,"7sus","000001000000"],
["m7 2 7",3,"7","101000000000"],
["m7 2 7",5,"7","101001000010"],
["m7 2 7",-3,"m7b5","010000100010"],
["m7 2 7",-4,"7","001000010100"],
["m7 -2 7",-1,"7b13","000000001000"],
["m7 -2 7",4,"dim","000100000000"],
["m7 -2 7",5,"7","000000000110"],
["m7 2 7b9",-3,"m7b5","000001000000"],
["m7 2 dim",5,"7","000001000000"],
["m7 2 dim",6,"7","000010000000"],
["m7 2 dim",3,"dim","000000010000"],
["m7 2 m7",5,"7sus","000001000000"],
["m7 2 m7",1,"dim","011010101101"],
["m7 2 m7",0,"m7b5","101101000010"],
["m7 2 m7",3,"mMaj7","101000010101"],
["m7 2 m7",-4,"7","010010010101"],
["m7 2 m7",5,"7","101011010010"],
["m7 -2 m7",6,"7b13","000010000000"],
["m7 -2 m7",-2,"7","000000000100"],
["m7 -2 m7",-1,"7b13","000000001000"],
["m7 -2 m7",5,"7sus","000001000000"],
["m7 -2 m7",-2,"7b13","001000000000"],
["m7 -2 m7",-1,"m7","000000000100"],
["m7 -2 m7",4,"7","000110100000"],
["m7 -2 m7",0,"dim","100100000000"],
["m7 -2 m7",-1,"7","010010000100"],
["m7 -2 m7",5,"7","001000010010"],
["m7 2 m7b5",3,"7b13","000010000000"],
["m7 2 m7b5",5,"7b9","010000000000"],
["m7 2 m7b5",5,"m7","000001000000"],
["m7 2 m7b5",-2,"7b13","000000000010"],
["m7 2 m7b5",3,"dim","000010100000"],
["m7 2 m7b5",6,"7","000000000100"],
["m7 2 m7b5",0,"m7b5","101000000010"],
["m7 2 m7b5",-1,"7","000010010101"],
["m7 2 m7b5",3,"7","101010110000"],
["m7 2 m7b5",5,"7b13","101011000010"],
["m7 2 m7b5",1,"7","011000101100"],
["m7 2 m7b5",-3,"m7b5","110101100010"],
["m7 2 m7b5",5,"7","101011010110"],
["m7 2 m7b5",-4,"7","111010110101"],
["m7 -2 Maj",-2,"dim","001000010001"],
["m7 -2 Maj",0,"m7b5","101101000000"],
["m7 -2 Maj",5,"7","100010010110"],
["m7 -2 Maj",3,"mMaj7","001000110101"],
["m7 3 7",2,"m7","100000000000"],
["m7 3 7",5,"7","001000000110"],
["m7 -3 7",5,"7","001000000100"],
["m7 -3 7",4,"m7b5","000100000010"],
["m7 3 7#11",5,"7","000000000010"],
["m7 3 7alt",5,"7","000000010000"],
["m7 -3 7alt",-1,"7","000000000100"],
["m7 -3 7alt",-4,"7","100000000000"],
["m7 3 m7",3,"7b13","000010000000"],
["m7 3 m7",3,"m7b5","100000000000"],
["m7 3 m7",2,"m7","100100011010"],
["m7 3 m7",0,"mMaj7","101100010000"],
["m7 3 m7",5,"7","100011010110"],
["m7 3 m7",0,"m7b5","101101100010"],
["m7 3 m7",3,"mMaj7","101100010101"],
["m7 -3 m7",5,"7","001000010110"],
["m7 -3 m7b5",-3,"7b13","000000000010"],
["m7 -3 m7b5",5,"7sus","000001000000"],
["m7 -3 m7b5",1,"7b9","010000000000"],
["m7 -3 m7b5",6,"7b9","010000000000"],
["m7 -3 m7b5",6,"7b13","000000001000"],
["m7 -3 m7b5",1,"7","010010100000"],
["m7 -3 m7b5",-4,"7","001010110100"],
["m7 -3 m7b5",6,"7","010010101101"],
["m7 -3 m7b5",5,"7","101011010110"],
["m7 -3 m7b5",0,"m7b5","101101010000"],
["m7 -3 m7b5",0,"mMaj7","101101010100"],
["m7 3 Maj",1,"7","000000110001"],
["m7 3 mMaj7",2,"m7","101000001010"],
["m7 3 mMaj7",3,"Maj","101010000101"],
["m7 -4 7",-1,"dim","000000001000"],
["m7 -4 7",1,"7b9","010000000000"],
["m7 -4 7",0,"7b13","101000000010"],
["m7 -4 7",-1,"7","000010000000"],
["m7 -4 7",5,"7b13","001000000010"],
["m7 -4 7",3,"7","100000000100"],
["m7 -4 7",6,"7","000000100100"],
["m7 -4 7",0,"m7b5","101000000010"],
["m7 -4 7",1,"7","011010100000"],
["m7 -4 7",3,"mMaj7","101000010101"],
["m7 -4 7",5,"7","101001010110"],
["m7 -4 7",-3,"m7b5","000101100010"],
["m7 -4 7#11",5,"7","000000000110"],
["m7 -4 7#11",-3,"m7b5","000001100000"],
["m7 -4 7alt",-1,"7","000010000000"],
["m7 -4 7alt",0,"7b13","100000000000"],
["m7 -4 7alt",5,"7b13","001000000010"],
["m7 -4 7alt",3,"7","100000000100"],
["m7 -4 7alt",6,"7","000000100100"],
["m7 -4 7alt",0,"m7b5","101000000000"],
["m7 -4 7alt",3,"mMaj7","100000000101"],
["m7 -4 7alt",1,"7","011000100000"],
["m7 -4 7alt",5,"7","001000010010"],
["m7 -4 7alt",-3,"m7b5","000101100010"],
["m7 4 dim",5,"7","001000010101"],
["m7 -4 dim",-1,"7b13","000000001000"],
["m7 -4 dim",5,"7","001000000000"],
["m7 4 m7",5,"7","001000000000"],
["m7 -4 m7",5,"7","000000010010"],
["m7 -4 m7",-2,"7sus","100000000000"],
["m7 -4 m7",5,"7sus","000001000000"],
["m7 -4 m7",-1,"7","000010000001"],
["m7 4 m7b5",1,"7","001000101000"],
["m7 4 m7b5",5,"7","100010000010"],
["m7 -4 Maj",-4,"dim","001000000000"],
["m7 -4 Maj",5,"7","001000000010"],
["m7 -4 Maj",-3,"m7b5","000100100010"],
["m7 -4 mMaj7",5,"7","000000000010"],
["m7 -4 mMaj7",-3,"m7b5","000000100000"],
["m7 5 7",2,"7b13","000000001000"],
["m7 5 7",6,"7","000010000000"],
["m7 5 7",5,"7b9","000001000000"],
["m7 5 7",0,"m7b5","000000000010"],
["m7 5 7",5,"m7b5","000001000000"],
["m7 -5 7",2,"m7b5","100100000000"],
["m7 -5 7",1,"7","011010000000"],
["m7 -5 7",-4,"7","000000110100"],
["m7 -5 7",5,"7","101000000110"],
["m7 5 7#11",5,"7b13","001000000000"],
["m7 5 7#11",2,"7b13","000000010010"],
["m7 5 7#11",6,"7","000010000100"],
["m7 5 7b13",6,"7","000000100000"],
["m7 -5 7b9",-1,"dim","010000000000"],
["m7 -5 7b9",2,"m7","100101000000"],
["m7 5 7sus",6,"7b13","000000001000"],
["m7 5 7sus",1,"dim","011000000000"],
["m7 5 7sus",6,"7","010010101101"],
["m7 5 dim",-1,"7","000010000000"],
["m7 5 dim",6,"7","000010000000"],
["m7 5 dim",5,"7b9","000000000010"],
["m7 5 dim",3,"dim","000000010000"],
["m7 5 dim",5,"7","000001000010"],
["m7 -5 dim",5,"7","000000000010"],
["m7 5 m7",-1,"7b9","010000000000"],
["m7 5 m7",6,"7b13","000000001000"],
["m7 5 m7",6,"7b9","010000000000"],
["m7 5 m7",5,"7b13","001000000000"],
["m7 5 m7",0,"dim","101110000000"],
["m7 5 m7",-1,"7","0100101100017"],
["m7 5 m7",0,"7","101010010110"],
["m7 5 m7",0,"mMaj7","1011011010000"],
["m7 5 m7",6,"7","0111101001014"],
["m7 5 m7",5,"7","1010110101101"],
["m7 -5 m7",2,"7","000000010000"],
["m7 -5 m7",-4,"7","000000110000"],
["m7 -5 m7",5,"7sus","001000010010"],
["m7 -5 m7",0,"mMaj7","001100010100"],
["m7 -5 m7",5,"7","001010010110"],
["m7 5 m7b5",-2,"7sus","100000000000"],
["m7 5 m7b5",5,"7","001000000110"],
["m7 -5 m7b5",1,"7","010000000000"],
["m7 -5 m7b5",5,"7","000000000100"],
["m7 -5 Maj",-2,"dim","000000000100"],
["m7 -5 Maj",5,"7","001000000000"],
["m7 6 7",5,"7","101000010110"],
["m7 6 7",1,"7","011010110101"],
["m7 6 7alt",1,"7b9","010000000000"],
["m7 6 7alt",1,"7","011010110101"],
["m7 6 7alt",5,"7","101000010110"],
["m7 6 m7",5,"7","000000000010"],
["m7b5 -1 7",5,"7","100000000000"],
["m7b5 -1 7",1,"7","100010000001"],
["m7b5 -1 7",-4,"7","000010010100"],
["m7b5 -1 7",5,"7b9","110000010010"],
["m7b5 -1 7",5,"m7b5","110000000010"],
["m7b5 -1 7",6,"7","100010110100"],
["m7b5 -1 7#11",5,"7","100000000000"],
["m7b5 -1 7alt",1,"7b9","010000000000"],
["m7b5 -1 7alt",0,"7b13","000000001000"],
["m7b5 -1 7alt",-4,"7","100000110100"],
["m7b5 -1 7alt",5,"m7b5","1100000000100"],
["m7b5 -1 7alt",1,"7","111010010000"],
["m7b5 -1 7alt",5,"7b9","1100000010110"],
["m7b5 -1 7alt",6,"7","10000101101008"],
["m7b5 -1 m7",-1,"mMaj7","000000000101"],
["m7b5 -2 m7",-1,"7","000010000000"],
["m7b5 -2 m7",3,"m7b5","000100000000"],
["m7b5 -2 Maj",5,"7","000000110000"],
["m7b5 -2 Maj",-1,"7","110110000000"],
["m7b5 -2 Maj",5,"7sus","000000110110"],
["m7b5 -2 Maj",-3,"dim","001101100000"],
["m7b5 3 m7",-1,"Maj","000000010000"],
["m7b5 -3 Maj",-1,"7","000010000000"],
["m7b5 5 7",5,"7b13","000000001000"],
["m7b5 5 7",1,"7b13","000010000000"],
["m7b5 5 7",-1,"7b9","010000000000"],
["m7b5 5 7",0,"7","000000000010"],
["m7b5 5 7",2,"7b13","000010000010"],
["m7b5 5 7",0,"m7","000101000000"],
["m7b5 5 7",6,"7","100010110100"],
["m7b5 5 7",-1,"7","011010110101"],
["m7b5 5 7",5,"7sus","1100011010010"],
["m7b5 5 7#11",-1,"7","000000100000"],
["m7b5 5 7b9",-1,"7b9","010000000000"],
["m7b5 5 7b9",6,"7b13","000000001000"],
["m7b5 5 7b9",-3,"dim","001101100100"],
["m7b5 5 7b9",5,"7","000000010110"],
["m7b5 5 7b9",-1,"7","1100110010004"],
["m7b5 5 7b9",6,"7","1000111101104"],
["m7b5 5 7sus",-1,"7","000000100000"],
["m7b5 5 m7",6,"7","000000000001"],
["m7b5 5 m7",-1,"7","000010100100"],
["m7b5 5 m7b5",4,"7","001000000000"],
["m7b5 5 m7b5",6,"7","000000000100"],
["m7b5 5 m7b5",-1,"7","000010010000"],
["Maj 0 7",-4,"7b13","000010000000"],
["Maj 0 7",-3,"7b13","000100000000"],
["Maj 0 7",-1,"7","100000010000"],
["Maj 0 7",1,"m7b5","000001000010"],
["Maj 0 7",3,"7","000000000100"],
["Maj 0 7",1,"7b13","010100000001"],
["Maj 0 7",6,"7","100000100000"],
["Maj 0 7",-2,"dim","001000101100"],
["Maj 0 7",4,"m7","100001010000"],
["Maj 0 7",-5,"m7b5","001000000100"],
["Maj 0 7",-5,"m7","101001010100"],
["Maj 0 7alt",1,"7b13","000001000000"],
["Maj 0 7alt",-1,"7","000000010000"],
["Maj 0 7alt",6,"7","100000000000"],
["Maj 0 7alt",1,"7sus","000001000000"],
["Maj 0 7alt",-2,"dim","000000001000"],
["Maj 0 7alt",1,"m7b5","000001000000"],
["Maj 0 7sus",0,"7","000010000000"],
["Maj 0 dim",0,"m7","000000000100"],
["Maj 0 m7",3,"7b13","000010000000"],
["Maj 0 m7",1,"7b9","010000000000"],
["Maj 0 m7",1,"dim","000100100001"],
["Maj 0 m7",-5,"7","000000000100"],
["Maj 0 m7",-3,"m7","000100001000"],
["Maj 0 m7",0,"dim","101001010101"],
["Maj 0 m7",-5,"7b13","100011010010"],
["Maj 0 m7",1,"7","010010101011"],
["Maj 0 m7",0,"mMaj7","1011010100104"],
["Maj 0 mMaj7",-3,"m7","101101010110"],
["Maj 1 7",-4,"7","000010000000"],
["Maj 1 7",2,"m7","000001010100"],
["Maj 1 7",-3,"m7","101100000010"],
["Maj -1 7",-1,"7sus","100000000000"],
["Maj -1 7",0,"m7b5","100000000000"],
["Maj -1 7",6,"m7b5","010001100010"],
["Maj -1 7",0,"7","100010010101"],
["Maj 1 7alt",-4,"7","000010000001"],
["Maj 1 7alt",2,"m7","001001010100"],
["Maj 1 7alt",-3,"m7","101100010010"],
["Maj -1 7alt",0,"7","000000010000"],
["Maj -1 7alt",6,"m7b5","010000100000"],
["Maj 1 dim",0,"7b13","000010000000"],
["Maj 1 dim",0,"mMaj7","000000110100"],
["Maj 1 dim",4,"m7","100100000010"],
["Maj 1 dim",-2,"7","001010101100"],
["Maj 1 dim",-3,"m7","100111110110"],
["Maj 1 m7",2,"7b13","100010000000"],
["Maj 1 m7",0,"7","000010110000"],
["Maj 1 m7",4,"7","100000010010"],
["Maj 1 m7",5,"7","001000100110"],
["Maj -1 m7",4,"7b9","100000000000"],
["Maj -1 m7",5,"7b9","010000000000"],
["Maj -1 m7",-1,"7b13","100000000000"],
["Maj -1 m7",4,"7","100000000000"],
["Maj -1 m7",2,"7sus","000001010000"],
["Maj -1 m7",4,"7b13","100000001000"],
["Maj -1 m7",0,"7sus","100000010010"],
["Maj -1 m7",0,"m7","100000000000"],
["Maj -1 m7",5,"m7","000000010000"],
["Maj -1 m7",2,"7b13","001000001000"],
["Maj -1 m7",5,"7sus","001000010000"],
["Maj -1 m7",4,"7sus","100000000000"],
["Maj -1 m7",0,"7b13","100010000000"],
["Maj -1 m7",0,"mMaj7","000000010101"],
["Maj -1 m7",-2,"7","001000000100"],
["Maj -1 m7",5,"7","001010110001"],
["Maj -1 m7",6,"m7b5","010100100010"],
["Maj -1 m7",0,"7","100010010111"],
["Maj 1 m7b5",5,"7b13","000000000010"],
["Maj 1 m7b5",5,"7","011010010000"],
["Maj 1 m7b5",-2,"7","001000100101"],
["Maj 1 m7b5",-3,"m7","100100110010"],
["Maj -1 m7b5",-2,"7","001000100000"],
["Maj -1 m7b5",0,"7","000010010110"],
["Maj -1 m7b5",5,"7","101011110100"],
["Maj -1 m7b5",5,"Maj","101010110101"],
["Maj 1 Maj",2,"7b13","001000000000"],
["Maj 1 Maj",2,"7","001000000000"],
["Maj 1 Maj",-4,"7","000010000001"],
["Maj -1 Maj",6,"7","100000100000"],
["Maj -1 Maj",0,"7b13","000010000000"],
["Maj -1 Maj",2,"7b13","001000000000"],
["Maj -1 Maj",5,"7","001000110000"],
["Maj 2 7",-3,"m7b5","000100000000"],
["Maj 2 7",-2,"7","001010000100"],
["Maj 2 7",-3,"m7","100101010010"],
["Maj 2 7",5,"7","001010010111"],
["Maj -2 7",4,"7b13","100000000000"],
["Maj -2 7",0,"7sus","000000010000"],
["Maj -2 7",-5,"7","000000000100"],
["Maj -2 7",-1,"m7","000000010000"],
["Maj -2 7",-3,"m7b5","000100000000"],
["Maj -2 7",5,"7sus","000011010000"],
["Maj -2 7",4,"7","001000010010"],
["Maj -2 7",-1,"m7b5","000101100000"],
["Maj -2 7",5,"Maj","101000010101"],
["Maj -2 7",5,"7","111010110101"],
["Maj -2 7",-3,"m7","101101011110"],
["Maj 2 7#11",-3,"m7","000000000010"],
["Maj -2 7#11",-3,"m7","000000010000"],
["Maj -2 7alt",-3,"m7b5","000100000000"],
["Maj -2 7alt",4,"7","100000010000"],
["Maj -2 7alt",5,"7sus","000010010000"],
["Maj -2 7alt",-1,"m7b5","010001000000"],
["Maj -2 7alt",5,"Maj","001000010001"],
["Maj -2 7alt",5,"7","001010110001"],
["Maj -2 7alt",-3,"m7","101100010010"],
["Maj 2 7b9",-3,"m7","000000010010"],
["Maj 2 7sus",-3,"m7","000000000010"],
["Maj 2 7sus",1,"m7b5","000000100000"],
["Maj -2 dim",-2,"7","000010000000"],
["Maj -2 dim",-3,"m7","000001000000"],
["Maj -2 dim",4,"m7","000000000010"],
["Maj -2 dim",0,"mMaj7","000000000100"],
["Maj 2 m7",-2,"7b13","000010000000"],
["Maj 2 m7",2,"7","000000000100"],
["Maj 2 m7",-4,"7b9","010000000000"],
["Maj 2 m7",3,"dim","010110101100"],
["Maj 2 m7",5,"7","000010000000"],
["Maj 2 m7",2,"7b13","001010000110"],
["Maj 2 m7",-3,"7b13","101000010100"],
["Maj 2 m7",-4,"7","010010100010"],
["Maj 2 m7",1,"dim","010111111011"],
["Maj 2 m7",-3,"m7","1001000100110"],
["Maj 2 m7",-5,"m7b5","1010110001011"],
["Maj 2 m7",3,"7","0101010101104"],
["Maj -2 m7",2,"7","000001000000"],
["Maj -2 m7",6,"7","000000100000"],
["Maj -2 m7",0,"dim","100010010100"],
["Maj -2 m7",6,"m7b5","000000100000"],
["Maj -2 m7",0,"7","100000000000"],
["Maj -2 m7",-3,"m7","000100000010"],
["Maj 2 m7b5",3,"7","000010000101"],
["Maj 2 m7b5",5,"7","000000010101"],
["Maj 2 m7b5",-3,"m7","000101010010"],
["Maj 3 7",1,"7b13","000000000010"],
["Maj 3 7",4,"7","000000010000"],
["Maj 3 7",-4,"m7b5","000100000000"],
["Maj 3 7",4,"7b13","000000001000"],
["Maj 3 7",0,"7b13","100010000000"],
["Maj 3 7",0,"7","100010010000"],
["Maj 3 7",4,"m7b5","100101001000"],
["Maj 3 7",-3,"7","101000010010"],
["Maj 3 7",4,"m7","100100010000"],
["Maj 3 7",-2,"7","001000100100"],
["Maj 3 7",-3,"m7","001000010010"],
["Maj -3 7",-2,"7b13","000000001000"],
["Maj -3 7",-2,"7b9","010000000000"],
["Maj -3 7",5,"7","001010110100"],
["Maj -3 7",4,"m7","100101010100"],
["Maj -3 7",0,"7","110010010100"],
["Maj -3 7",-2,"7","011110100101"],
["Maj 3 7alt",1,"7b13","000000000010"],
["Maj 3 7alt",2,"7b13","000010000000"],
["Maj 3 7alt",-4,"m7b5","000100000000"],
["Maj 3 7alt",4,"7","000000010000"],
["Maj 3 7alt",4,"7b13","000000001010"],
["Maj 3 7alt",0,"7b13","100110010001"],
["Maj 3 7alt",-3,"m7","001100110010"],
["Maj 3 7alt",4,"m7","100100011011"],
["Maj 3 7alt",0,"7","100110010000"],
["Maj 3 7alt",-3,"7","101000110010"],
["Maj 3 7alt",4,"m7b5","101101011001"],
["Maj 3 7alt",-2,"7","011011100101"],
["Maj -3 7b9",4,"m7","100000000000"],
["Maj -3 7b9",-2,"7","001000100001"],
["Maj -3 7b9",4,"m7b5","100001001000"],
["Maj 3 dim",-2,"dim","001000000000"],
["Maj 3 dim",3,"7b13","000010101000"],
["Maj 3 dim",4,"m7","101101010010"],
["Maj 3 dim",-3,"m7","101100010110"],
["Maj -3 dim",5,"Maj","100000000000"],
["Maj -3 dim",3,"7b13","000010000000"],
["Maj -3 dim",-3,"m7","000000000010"],
["Maj -3 dim",4,"m7","000100000010"],
["Maj 3 m7",4,"m7","100100010010"],
["Maj 3 m7",2,"m7","100111000110"],
["Maj 3 m7",-3,"m7","001101011110"],
["Maj -3 m7",5,"7b13","000010000000"],
["Maj -3 m7",-2,"7b9","010000000000"],
["Maj -3 m7",4,"7b9","110000000000"],
["Maj -3 m7",2,"dim","101100000110"],
["Maj -3 m7",4,"7b13","001000001000"],
["Maj -3 m7",-5,"7","1011000000000"],
["Maj -3 m7",-3,"dim","100100000010"],
["Maj -3 m7",1,"7b13","000100101001"],
["Maj -3 m7",-1,"m7b5","00000110011040"],
["Maj -3 m7",0,"7sus","11000100100000"],
["Maj -3 m7",4,"7","1101000011010"],
["Maj -3 m7",3,"7","0010111100104"],
["Maj -3 m7",-5,"7sus","1100001000100"],
["Maj -3 m7",5,"7sus","10100100110004"],
["Maj -3 m7",5,"7","00111101110100"],
["Maj -3 m7",-2,"7","011101011111112"],
["Maj -3 m7b5",-3,"m7","000100000000"],
["Maj -3 m7b5",0,"m7","101000000000"],
["Maj 3 Maj",-2,"dim","000000000100"],
["Maj -3 Maj",0,"7b13","000010000000"],
["Maj -3 mMaj7",4,"7","000010000000"],
["Maj 4 7",-2,"7b13","000000001000"],
["Maj 4 7",-5,"7","000000000100"],
["Maj 4 7",5,"7","011000111100"],
["Maj 4 7",-1,"m7b5","000101001000"],
["Maj 4 7",-2,"7","011110100000"],
["Maj -4 7",3,"7b9","010000000000"],
["Maj -4 7",3,"7sus","000001000000"],
["Maj -4 7",-2,"7","000010100100"],
["Maj -4 7",3,"7","000010000101"],
["Maj -4 7",0,"7sus","001010010000"],
["Maj -4 7",-3,"m7","001101010010"],
["Maj 4 7#11",5,"7","001000000000"],
["Maj -4 7alt",-2,"7","000000100000"],
["Maj -4 7alt",3,"7","010000000000"],
["Maj -4 7alt",0,"7sus","000010000000"],
["Maj -4 7alt",-3,"m7","000000010000"],
["Maj 4 7b9",-1,"m7b5","000001000000"],
["Maj 4 dim",4,"m7","000100000000"],
["Maj 4 m7",0,"7b13","000010000000"],
["Maj 4 m7",5,"7b9","010000000000"],
["Maj 4 m7",-2,"7b9","010000000000"],
["Maj 4 m7",2,"dim","000000000100"],
["Maj 4 m7",-3,"m7","001100010010"],
["Maj 4 m7",-5,"7sus","101001010000"],
["Maj 4 m7",2,"m7","101001000110"],
["Maj 4 m7",-2,"7","011010110100"],
["Maj 4 m7",5,"7","101010110100"],
["Maj -4 m7",-3,"m7","100100000010"],
["Maj 4 m7b5",-2,"7b13","000000001000"],
["Maj 4 m7b5",2,"7","000000000100"],
["Maj 4 m7b5",-2,"7b9","010000000000"],
["Maj 4 m7b5",5,"7b9","010000000000"],
["Maj 4 m7b5",-3,"7b13","001000000000"],
["Maj 4 m7b5",0,"dim","000100100000"],
["Maj 4 m7b5",2,"7b9","000000010100"],
["Maj 4 m7b5",2,"7b13","001001001010"],
["Maj 4 m7b5",-5,"m7b5","101100001000"],
["Maj 4 m7b5",0,"7b13","100011110010"],
["Maj 4 m7b5",3,"7","100010110100"],
["Maj 4 m7b5",-5,"7b13","101111001001"],
["Maj 4 m7b5",4,"m7","101101110000"],
["Maj 4 m7b5",-2,"7","111001010100"],
["Maj 4 m7b5",2,"m7","001101001110"],
["Maj 4 m7b5",5,"7","001011110011"],
["Maj 4 Maj",5,"Maj","000000000001"],
["Maj -4 Maj",-3,"m7","000000000010"],
["Maj -4 Maj",3,"7","000010000100"],
["Maj 5 7",5,"7b13","001000000000"],
["Maj 5 7",1,"7b13","000000100000"],
["Maj 5 7",0,"7b13","101010100101"],
["Maj 5 7",0,"mMaj7","101000100101"],
["Maj 5 7",0,"7","100011110100"],
["Maj 5 7",6,"m7b5","100101101010"],
["Maj -5 7",2,"7b13","001000000000"],
["Maj -5 7",3,"7b13","000010000000"],
["Maj -5 7",1,"7b13","000000001001"],
["Maj -5 7",1,"7","000000100001"],
["Maj -5 7",6,"dim","000100100010"],
["Maj -5 7",-4,"7","000010000101"],
["Maj -5 7",2,"m7","001001010010"],
["Maj 5 7#11",0,"mMaj7","101000010000"],
["Maj 5 7#11",6,"m7b5","010100101100"],
["Maj 5 7#11",-3,"m7","101101100010"],
["Maj 5 7alt",1,"7b13","000000100000"],
["Maj 5 7alt",0,"7","100010110101"],
["Maj 5 7alt",0,"7b13","101010100001"],
["Maj 5 7alt",0,"mMaj7","101000110000"],
["Maj 5 7alt",6,"m7b5","110101101010"],
["Maj -5 7b13",-3,"m7","100000000000"],
["Maj -5 7sus",-5,"7b13","100000000000"],
["Maj -5 7sus",0,"7b13","100000000000"],
["Maj -5 7sus",-2,"7","000000100100"],
["Maj -5 7sus",3,"7","000010000100"],
["Maj -5 7sus",5,"7","001000010001"],
["Maj -5 7sus",-3,"m7","000100010010"],
["Maj -5 dim",0,"m7","000000010000"],
["Maj -5 dim",4,"m7","000100000010"],
["Maj -5 dim",0,"mMaj7","100000010000"],
["Maj -5 dim",-2,"7","001010000100"],
["Maj -5 dim",-3,"m7","000101000010"],
["Maj 5 m7",6,"7b13","000000001000"],
["Maj 5 m7",0,"7sus","000001000000"],
["Maj 5 m7",3,"dim","000010000000"],
["Maj 5 m7",-4,"7b13","000010001000"],
["Maj 5 m7",6,"7b9","010000000000"],
["Maj 5 m7",6,"m7b5","000001100000"],
["Maj 5 m7",6,"7","010000100011"],
["Maj 5 m7",1,"dim","000100100001"],
["Maj 5 m7",4,"m7","100100010010"],
["Maj 5 m7",2,"m7","100001000010"],
["Maj 5 m7",-3,"m7","001100010010"],
["Maj 5 m7",0,"7","101011010000"],
["Maj -5 m7",-5,"7b13","001000000000"],
["Maj -5 m7",-4,"7b13","000010001000"],
["Maj -5 m7",6,"7","000010100001"],
["Maj -5 m7",0,"7b13","000010010000"],
["Maj -5 m7",2,"m7b5","100000001010"],
["Maj -5 m7",6,"dim","010111101011"],
["Maj -5 m7",2,"m7","100100000100"],
["Maj -5 m7",-4,"7","011010100101"],
["Maj -5 m7",1,"7","000010100111"],
["Maj -5 m7",-3,"m7","011101010010"],
["Maj -5 m7b5",0,"7b13","000010000000"],
["Maj -5 m7b5",-2,"7","000000100110"],
["Maj 5 Maj",-2,"7b9","010000000000"],
["Maj 5 Maj",6,"7","000001101000"],
["Maj 5 Maj",-2,"7","011000000100"],
["Maj 5 Maj",0,"7","100010010101"],
["Maj 5 Maj",0,"7b13","101010010100"],
["Maj 5 Maj",2,"7b13","101001010110"],
["Maj -5 Maj",2,"7b13","000000001000"],
["Maj -5 Maj",2,"7sus","000001000000"],
["Maj -5 Maj",-2,"dim","100001001100"],
["Maj -5 Maj",2,"7","010011001010"],
["Maj -5 Maj",0,"mMaj7","000100110011"],
["Maj -5 Maj",5,"7","011001110010"],
["Maj 5 mMaj7",6,"m7b5","010000000000"],
["Maj 5 mMaj7",5,"Maj","000000010000"],
["Maj 6 7",-3,"m7b5","010000000000"],
["Maj 6 7",2,"7b13","000000010010"],
["Maj 6 7",3,"7b13","000011100000"],
["Maj 6 7",1,"m7b5","000100111000"],
["Maj 6 7",-5,"m7","110001000100"],
["Maj 6 7",0,"7","100010011110"],
["Maj 6 7#11",1,"m7b5","000000100000"],
["Maj 6 7alt",2,"7b13","000000000010"],
["Maj 6 7alt",3,"7b13","010000000000"],
["Maj 6 7alt",1,"m7b5","000100100000"],
["Maj 6 7alt",0,"7","100010010000"],
["Maj 6 7alt",-5,"m7","100001000100"],
["Maj 6 7b9",1,"m7b5","000000100000"],
["Maj 6 dim",2,"7b13","000000000010"],
["Maj 6 dim",-3,"7b13","101000000000"],
["Maj 6 dim",-3,"m7","101000000000"],
["Maj 6 dim",4,"m7","000001010000"],
["Maj 6 dim",5,"Maj","000000010000"],
["Maj 6 m7",-3,"7","000000010010"],
["Maj 6 m7",0,"7","100010010000"],
["Maj 6 m7",5,"7","001010110100"],
["Maj 6 m7",-3,"m7","101101010010"],
["Maj 6 m7b5",1,"7alt","000000100000"],
["Maj 6 m7b5",-5,"m7b5","100000000000"],
["Maj 6 m7b5",-5,"7sus","0000010000000"],
["Maj 6 m7b5",-5,"7","101000000100"],
["Maj 6 m7b5",3,"7","000010100100"],
["Maj 6 m7b5",-2,"7","001011100100"],
["Maj 6 m7b5",0,"7","101010010100"],
["Maj 6 m7b5",5,"7","001010110111"],
["Maj 6 m7b5",-3,"m7","101101110010"],
["mMaj7 0 7b13",5,"7","000000000100"],
["mMaj7 0 7b13",0,"m7","001000000000"],
["mMaj7 -1 7",5,"7","000000010000"],
["mMaj7 -1 7",0,"m7","100000000000"],
["mMaj7 -1 m7",0,"m7","001000000100"],
["mMaj7 -1 m7",5,"7","001011110100"],
["mMaj7 2 m7",-3,"m7b5","000000001000"],
["mMaj7 2 m7b5",0,"m7","000000010000"],
["mMaj7 2 m7b5",-3,"m7b5","000000100010"],
["mMaj7 3 7",5,"7","001000000010"],
["mMaj7 -3 7",5,"7","000010000000"],
["mMaj7 -3 m7b5",0,"m7","001001010100"],
["mMaj7 -4 7",0,"m7","000100010000"],
["mMaj7 -4 7",-3,"m7b5","000000100010"],
["mMaj7 4 m7b5",1,"7b13","000000100000"],
["mMaj7 5 7",0,"m7","101100010100"],
["mMaj7 5 7alt",0,"m7","100000000000"],
["mMaj7 -5 m7",5,"7","000000010000"],
["mMaj7 -5 m7",0,"m7","100000000000"],
["mMaj7 -5 Maj",-2,"dim","100000000101"],
["mMaj7 -5 Maj",2,"7","100001010110"],
];

var goodchords = [
["",0],
["",0],
["",0],
["",0],
["",0],
["",0],
["",0],
["",0],
["",0],
["",0],
["",0],
["",0],
["",0],
["",0],
["",0],
["",0],
["",0],
["",0],
["",0],
["",0],
];

function convertalt(alt, altconvert){
		alt = chordUCase (alt);
		for(var i=0; i<altconvert.length; i++){
			if(altconvert[i][0] == alt){
				return altconvert[i][1];
		}
	}	
}

// convert note to uppercase
function noteUCase(s){
	s = s.toUpperCase();
	if (s.substr(1,1) == "B"){
		s = s.substr(0,1) + "b";
	}	
	return s;
}
function chordUCase(s){
	var u = s.substr(0,1);
	u = u.toUpperCase();
	s = u + s.substr(1);	
	return s;
}

function findSpread(count){
	var base = 1;
	while (count > base){
		base *= 2;
	}

	return base;
}

function soundsGood(newhead,melnote, sound){
		if (melnote == "") return 1;
		var umelnote = noteUCase(melnote);
		// //console.log ("umelnote = ", umelnote)	
		var i1 = findindex(newhead, heads);
		var i2 = findindex(umelnote, heads);
		var i3 = findDelta(i1, i2);
		// //console.log ("i1 = ", i1, "i2 = ", i2, "i3 = ", i3)
		// //console.log ("sound = ", sound)
		var r = sound.substr(i3,1);
		//console.log ("SoundsGood = ", r)
		if (r == "p") return 1;
		return r;
}

// collect the existing chord symbols and their segment.tick
var chordnames = new Array(512); 
// chordnames
// O: chordname
// 1: tick
// 2: head
// 3: tail
// 4: altflag
for(var i=0; i<128; i++) chordnames[i]=new Array();
var count = 0;
for (var track = 0; track < /*curScore.ntracks*/ 1; ++track) {
	var segment = curScore.firstSegment();
	while (segment) {
		var aCount = 0;
		var annotation = segment.annotations[aCount];

		while (annotation) {
			if (annotation.type == Element.HARMONY) {
				var harmony =  annotation;
				// //console.log('harmony.text = ' + harmony.text)
				var str = harmony.text;
				var n = str.search("/");
					if (n > -1){
						// //console.log('str = ' + str);
						str = str.substr(0,n);
						// //console.log('str = ' + str);
					}
				//str = str.replace("dim7","dim");
				chordnames[count][0] = str;
				chordnames[count][1] = segment.tick;
				var headlen = findheadlen(str);
				var hstr = str.substr(0,headlen);
				// make the root uppercase
				hstr = noteUCase(hstr);
				
				chordnames[count][2] = hstr;
				// Adapt alternate chord symbols
				if (str.length - headlen > 0) chordnames[count][3] = str.substr(headlen,(str.length - headlen))
				else chordnames[count][3] = "Maj";
				if (chordnames[count][3] == "mM7") chordnames[count][3] = "mMaj7"
				if (chordnames[count][3] == "dim7") chordnames[count][3] = "dim"
				// change alt to 7
				// but we need to keep the alt name for output to console log
				var n = chordnames[count][0].search("7alt");
					// set altflag to no
					chordnames[count][4] = "";
					if (n > -1){
						chordnames[count][4] = chordnames[count][0];
						chordnames[count][0] = convertalt(chordnames[count][0], altconvert);
						//var i1 = findindex(chordnames[count][2], heads);
						//var i2 = findnewdistance(i1, 6);
						//chordnames[count][2] = findhead(chordnames[count][2],i2, heads,sharpheads, flatheads)
						//var headlen = findheadlen(str);
						//var hstr = str.substr(0,headlen);
						//console.log ('chordnames[count][0] = ' + chordnames[count][0])
						chordnames[count][2] = chordnames[count][0].substr(0, chordnames[count][0].length -1)
						chordnames[count][3] = "7";
						
						//console.log ('chordnames[count][2] = ' + chordnames[count][2])
					}
				// //console.log("head -", chordnames[count][2],"-");
				// //console.log("tail -",str.substr(headlen,(str.length - headlen)),"-");
				// //console.log('harmony.text = ' + harmony.text);
				//console.log('time = ' + segment.tick);
				
				count++;					
			}
			annotation = segment.annotations[++aCount];
		}
		segment = segment.next;
	}
}
		// 1440 voor 3/4
		// 1920 voor 4/4
		var onemeasure = 1920;
		//add a whole note of time to the last measure
		//chordnames[count][1] = chordnames[count-1][1] + onemeasure;
		chordnames[count][1] = chordnames[count-1][1] + 480;
		
		// construct the chord pairs
		for (var c = 0; c < count-1; c++){
			var gap = chordnames[c+1][1] - chordnames[c][1];
			if (gap > mingap){
				var measure = chordnames[c][1]/onemeasure + 1;
				// NB Only head is uppercase not chordname[c][0]
				// //console.log("chord pair: " + chordnames[c][0] + " " + chordnames[c+1][0] + " m" + measure);
				var i1 = findindex(chordnames[c][2], heads);
				var i2 = findindex(chordnames[c+1][2],heads);
				var distance = i2 - i1;
				if (distance < -5) distance += 12;
				if (distance > 6) distance -= 12;
				
				// q = "tail1 distance tail2"
				var q = chordnames[c][3] + " " + distance + " " + chordnames[c+1][3];
				// //console.log("q =", q);
                 var goodcount = 0;
				
				// search for q in the embel table
				for (var d = 0; d < embel.length; d++){
                   
					// this stacks embellishments halfway between existing symbols
					if (embel[d][0] == q){
						
						var newdist = findnewdistance (i1,embel[d][1]);
						if (embel[d][2] == "dim")
							var newhead = finddimhead(newdist, dimheads);
						else
							var newhead = findhead(chordnames[c][2],newdist, heads, sharpheads, flatheads);

						var newchord = newhead + embel[d][2];
						//console.log("newchord = " + newchord)
						// is it a slash chord?
						// in the embel table the /base is notated as offset from the root in halftones
						var n = newchord.search("/");
						if (n > -1){
							//console.log("newchord = " + newchord)
							var slashi = newchord.substr(n+1, newchord.length - (n+1));
							var slashi2 = findnewdistance (slashi,newdist);
							//console.log("slashi = " + slashi)
							//console.log("slashi2 = " + slashi2)
							//console.log("find shead: newhead = " + newhead)
							var shead = findshead(newhead,slashi, slashi2,heads,sharpheads, flatheads, slashhead);
							//console.log("find shead: newhead = " + newhead)
							newchord = newchord.substr(0,n+1) + shead;
							//console.log("newchord = " + newchord)
						}
						var newtime = chordnames[c][1] + (gap/2);
						// not good here?
						//if (onemeasure == 1440 & gap == 1440){
						//	newtime = chordnames[c][1] + 960;
						//}
						// //console.log("newtime: ", newtime);
						setCursorToTime(cursor, newtime); 
						// search for newtime in melnotes
						var mfound = 0;
                        var melnote = "";
                        var melnotefound = 0;
                        for (var j = 0; j < mcount; j++ && (mfound == 0)){
                            if (melnotes[j][1] == newtime){
                                // //console.log("melnote: ", melnotes[j][0]);
                                melnote = melnotes[j][0];		
                                mfound = 1;
                                melnotefound = 1;
                            }
							// if no melody note at newtime
							// see if there is another melody note before the next chord
							// if so choose it for embel insert
							// otherwise use the prior note
                            if (j > 0 && melnotes[j][1] > newtime && mfound == 0){
                                if (melnotes[j][1] < chordnames[c+1][1]){
                                    melnote = melnotes[j][0];
                                    newtime = melnotes[j][1];
									//need to adjust gap here
									gap = (chordnames[c+1][1] - newtime) * 2;
									// console.log("new gap: ", gap);
                                    var result = setCursorToTime2(cursor, newtime);
									// //console.log("result: ", result); 
                                    // //console.log("cursor.tick: ", cursor.tick);
                                    // //console.log("melnotes[j][1] = ",melnotes[j][1]);
                                    // //console.log("chordnames[c+1][1] = ", chordnames[c+1][1]);
                                }
                                else {
                                    melnote = melnotes[j-1][0];
                                }
                                // //console.log("melnote: ", melnote);
                                mfound = 1;
                            }
                        }
						// //console.log("newchord: ", newchord);
						var result = soundsGood(newhead,melnote, embel[d][3]);
						// no cursor segment in tuplets
						if (result == 1 && cursor.tick != 0){
                            goodchords[goodcount][0] = newchord;
                            goodchords[goodcount++][1] = newtime;
                            // //console.log("goodcount: ", goodcount); 
							//var harmony = newElement(Element.HARMONY);
							//harmony.text = newchord;
							// //console.log("melnote: ", melnote);
							// //console.log("cursor.tick: ", cursor.tick);
							//cursor.add(harmony);
						}
					} // end of 1 embel found
				} // end of this chord pair
                // ////console.log("gap: ", gap);
                //console.log("goodcount: ", goodcount);
				
                // add the embellishments for this chord pair
				for (var z= 0; z < goodcount; z++){
                    // //console.log("goodchord: ", goodchords[z][0] + ' ' + goodchords[z][1]);
                    var harmony = newElement(Element.HARMONY);
					harmony.text = goodchords[z][0];
					// see if goodcount+1 prevents last embel chord on top of next fixed chord => NG
					var spread = findSpread(goodcount);
					var shift = gap/spread;
					//var maxshift = 480;
					//if (shift > maxshift) shift = maxshift;
					if (goodcount > maxembel) goodcount = maxembel;
					var offset =  newtime + (shift * z/2);
					//var offset =  newtime + (shift * z);
                    setCursorToTime(cursor, offset); 
					//console.log("offset: ", offset);
					//console.log("spread: ", spread);
					//console.log("newtime: ", newtime);
					//console.log("gap: ", gap);
					cursor.add(harmony);
                }
				// send this chord pair to console log
				var s = ""
				if (goodcount >0){
					if(chordnames[c][4] == "")
						s = chordUCase(chordnames[c][0]) + "-";
					else
						s = chordUCase(chordnames[c][4]) + "-";
					if(chordnames[c+1][4] == "")
						s += chordUCase(chordnames[c+1][0]) + ".mscz" + "   " + "m" + ((chordnames[c][1]/onemeasure) + 1) + "   " +melnote;
					else
						s += chordUCase(chordnames[c+1][4]) + ".mscz" + "   " + "m" + ((chordnames[c][1]/onemeasure) + 1) + "   " +melnote;
					
					//s = chordUCase(chordnames[c][0]) + "-" + chordUCase(chordnames[c+1][0]) + ".mscz" + "   " + "m" + ((chordnames[c][1]/onemeasure) + 1) + "   " +melnote;
					console.log(s);
				}
			} // end of this chord pair > mingap
		} // end of all chord pairs
		
		Qt.quit();
	}// end on Run
}
